﻿//
// MyDirectXControl.cpp
// Implementation of the MyDirectXControl class.
//

#include "pch.h"
#include "MyDirectXControl.h"

using namespace MyControlsLibrary;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Documents;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;

using namespace Windows::UI::Xaml::Media::Imaging;

using namespace Microsoft::WRL;
//using namespace Windows::Graphics::Display;
//using namespace Windows::UI::Core;
//using namespace Windows::System;
//using namespace Windows::Foundation;

// The Templated Control item template is documented at http://go.microsoft.com/fwlink/?LinkId=234235

MyDirectXControl::MyDirectXControl()
{
	DefaultStyleKey = "MyControlsLibrary.MyDirectXControl";
}

SurfaceImageSource^ MyDirectXControl::CreateImageSource(int width, int height)
{
	//Define the size of the shared surface by passing the height and width to 
	//the SurfaceImageSource constructor. You can also indicate whether the surface
	//needs alpha (opacity) support.
	SurfaceImageSource^  surfaceImageSource = ref new SurfaceImageSource(width, height, true);
	if(width <= 0 || height <= 0) return surfaceImageSource;
	

	//Get a pointer to ISurfaceImageSourceNative. Cast the SurfaceImageSource object
	//as IInspectable (or IUnknown), and call QueryInterface on it to get the underlying
	//ISurfaceImageSourceNative implementation. You use the methods defined on this 
	//implementation to set the device and run the draw operations.
	IInspectable* sisInspectable = (IInspectable*) reinterpret_cast<IInspectable*>(surfaceImageSource);
	sisInspectable->QueryInterface(__uuidof(ISurfaceImageSourceNative), (void **)&m_imageSourceNative);

	//Set the DXGI device by first calling D3D11CreateDevice and then passing the device and 
	//context to ISurfaceImageSourceNative::SetDevice. 

	// This flag adds support for surfaces with a different color channel ordering than the API default.
    // It is recommended usage, and is required for compatibility with Direct2D.
    UINT creationFlags = D3D11_CREATE_DEVICE_BGRA_SUPPORT;

#if defined(_DEBUG)
    // If the project is in a debug build, enable debugging via SDK Layers with this flag.
    creationFlags |= D3D11_CREATE_DEVICE_DEBUG;
#endif
	 // This array defines the set of DirectX hardware feature levels this app will support.
    // Note the ordering should be preserved.
    D3D_FEATURE_LEVEL featureLevels[] = 
    {
        D3D_FEATURE_LEVEL_11_1,
        D3D_FEATURE_LEVEL_11_0,
        D3D_FEATURE_LEVEL_10_1,
        D3D_FEATURE_LEVEL_10_0,
		D3D_FEATURE_LEVEL_9_3,
        D3D_FEATURE_LEVEL_9_2,
        D3D_FEATURE_LEVEL_9_1
    };

	HRESULT hr = D3D11CreateDevice(
			NULL,
			D3D_DRIVER_TYPE_HARDWARE,
			NULL,
			creationFlags,
			featureLevels,
			ARRAYSIZE(featureLevels),
			D3D11_SDK_VERSION,
			&m_d3dDevice,
			NULL,
			&m_d3dContext
			);	
	 if(FAILED(hr))
		 throw ref new COMException(hr);
	Microsoft::WRL::ComPtr<IDXGIDevice> dxgiDevice;
	// Obtain the underlying DXGI device of the Direct3D11.1 device.
	m_d3dDevice.As(&dxgiDevice);
	m_imageSourceNative->SetDevice(dxgiDevice.Get());

	return surfaceImageSource;
}

void MyDirectXControl::Draw(int width, int height)
{
	if(width <= 0 ||  height <= 0) return;
	ComPtr<IDXGISurface> surface;
	RECT updateRect = {0};
	updateRect.left = 0;
	updateRect.right = width;
	updateRect.top = 0;
	updateRect.bottom = height;
	POINT offset = {0};
	
	//Provide a pointer to IDXGISurface object to ISurfaceImageSourceNative::BeginDraw, and 
	//draw into that surface using DirectX. Only the area specified for update in the 
	//updateRect parameter is drawn. 
	//
	//This method returns the point (x,y) offset of the updated target rectangle in the offset
	//parameter. You use this offset to determine where to draw into inside the IDXGISurface.
	HRESULT beginDrawHR = m_imageSourceNative->BeginDraw(updateRect, &surface, &offset);
	if (beginDrawHR == DXGI_ERROR_DEVICE_REMOVED || beginDrawHR == DXGI_ERROR_DEVICE_RESET)
	{
			  // device changed
	}
	else
	{
		// draw to IDXGISurface (the surface paramater)
		// get D3D texture from surface returned by BeginDraw
		ComPtr<ID3D11Texture2D> d3DTexture;
		surface.As(&d3DTexture);
		// create render target view
		ComPtr<ID3D11RenderTargetView> m_renderTargetView;
		m_d3dDevice->CreateRenderTargetView(d3DTexture.Get(), nullptr, &m_renderTargetView);
		auto brush = (SolidColorBrush^)this->Background;
		float a = brush->Color.A/255.0f;
		float r = brush->Color.R/255.0f;
		float g = brush->Color.G/255.0f;
		float b = brush->Color.B/255.0f;
		 //const float clearColor[4] = { 0.071f, 0.04f, 0.561f, 1.0f };
		 const float clearColor[4] = { r, g, b, a };
		 m_d3dContext->ClearRenderTargetView(m_renderTargetView.Get(),clearColor);
	}

	// draw the map here

	//Call ISurfaceImageSourceNative::EndDraw to complete the bitmap. Pass this bitmap to an ImageBrush.
	m_imageSourceNative->EndDraw();
}

void MyDirectXControl::OnApplyTemplate() 
{
	if(m_MapSurfaceElement != nullptr)
	{	
		m_MapSurfaceElement->SizeChanged -= _sizeChangedToken;
		m_MapSurfaceElement = nullptr;
	}
	//Get template child for draw surface
	m_MapSurfaceElement = (Border^)GetTemplateChild("DrawSurface");
	if(m_MapSurfaceElement != nullptr)
	{
		int width = (int)m_MapSurfaceElement->ActualWidth;
		int height = (int)m_MapSurfaceElement->ActualHeight;
		m_imageSource = CreateImageSource(width, height);
		ImageBrush^ brush = ref new ImageBrush();
		brush->ImageSource = m_imageSource;
		m_MapSurfaceElement->Background = brush;
		_sizeChangedToken = m_MapSurfaceElement->SizeChanged += ref new SizeChangedEventHandler(this, &MyDirectXControl::OnMapSurfaceSizeChanged);
	}
}

//surface size changed handler
void MyDirectXControl::OnMapSurfaceSizeChanged(Platform::Object^ sender, Windows::UI::Xaml::SizeChangedEventArgs^ e)
{
	int width = (int)m_MapSurfaceElement->ActualWidth;
	int height = (int)m_MapSurfaceElement->ActualHeight;
	m_imageSource = CreateImageSource(width, height);
	Draw(width, height);
	ImageBrush^ brush = ref new ImageBrush();
	brush->ImageSource = m_imageSource;
	m_MapSurfaceElement->Background = brush;
}
//force a new redraw
void MyDirectXControl::Refresh()
{
	int width = (int)m_MapSurfaceElement->ActualWidth;
	int height = (int)m_MapSurfaceElement->ActualHeight;
	Draw(width, height);
}